/*******************************************************************************
 * Western Digital wd90c3x I/O
 *******************************************************************************
 *
   Copyright (C) 1995 by The Linux-GGI Project
 *
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************
 *

 * Revision 1.1  1997/02/03 15:06:35  bri
 * Major: IBM vga driver (initial version, taken from GGI 0.0.8)
 *
 *******************************************************************************
 * This code was derived from the following source of information:
 * [1]	XFree86 vga driver
 */

/* GGI includes */

#include "WD/wd90c3x_io.h"

/* Clock I/O functions */

void CLK_out8(struct kgi_display *dpy, kgiu8 value, kgiu8 reg)
{	
  kgiu8 regval;
  if ((reg != 0) | (value > 0xf))
    {	
      INTERNAL_ERROR;
    }
  
  regval = inbMISC();
  regval = ((regval & ~MISC_CLOCK_SEL_B0) | 
	    ((value & 0x01) ? MISC_CLOCK_SEL_B0 : 0));
  regval = ((regval & ~MISC_CLOCK_SEL_B1) | 
	    ((value & 0x02) ? MISC_CLOCK_SEL_B1 : 0));
  outbMISC(regval);

  regval = (inbGRC(GRC_UPGRC_LOCK) | GR0F_RLOCK); 
                                   /* Can't read rlock, so leave locked */
  outbGRC(((regval & ~(GR0F_WLOCK_MASK | GR0F_RLOCK)) | GR0F_WUNLOCK), 
	GRC_UPGRC_LOCK); 
  outbGRC(((inbGRC(GRC_VID_CONF) & ~GR0C_CLOCK_SEL_B2) |
	  ((value & 0x04) ? GR0C_CLOCK_SEL_B2 : 0)),
	GRC_VID_CONF);
  outbGRC(regval, GRC_UPGRC_LOCK);

  regval = inbSEQ(SEQ_UPSEQ_LOCK);
  outbSEQ(SR06_UNLOCK_UPSEQ,SEQ_UPSEQ_LOCK);
  outbSEQ(((inbSEQ(SEQ_MISCO4) & ~SR12_CLOCK_SEL_B3) |
	  ((value & 0x08) ? SR12_CLOCK_SEL_B3 : 0)),  
	  SEQ_MISCO4);
  outbSEQ(regval, SEQ_UPSEQ_LOCK);
}

kgiu8 CLK_in8(struct kgi_display *dpy, kgiu8 reg)
{
  kgiu8 result, lockreg;
  if (reg != 0)
    { 
      INTERNAL_ERROR;
    } 

  result = 0;

  result += (inbMISC() & MISC_CLOCK_SEL_B0) ? 1 : 0;
  result += (inbMISC() & MISC_CLOCK_SEL_B1) ? 2 : 0;

  lockreg = (inbGRC(GRC_UPGRC_LOCK) | GR0F_RLOCK); 
                                   /* Can't read rlock, so leave locked */
  outbGRC((lockreg & ~GR0F_RLOCK), GRC_UPGRC_LOCK);
  result += (inbGRC(GRC_VID_CONF) & GR0C_CLOCK_SEL_B2) ? 4 : 0;
  outbGRC(lockreg, GRC_UPGRC_LOCK);

  lockreg = inbSEQ(SEQ_UPSEQ_LOCK); 
  outbSEQ(SR06_UNLOCK_UPSEQ,SEQ_UPSEQ_LOCK);
  result += (inbSEQ(SEQ_MISCO4) & SR12_CLOCK_SEL_B3) ? 8 : 0; 
  outbSEQ(lockreg, SEQ_UPSEQ_LOCK);

  return(result);
}

/* Set paged linear register A  -- extra bit on wd90c33 */
static void outPR0A(kgiu16 page)
{
  kgiu8 regval /*, regval2 */;
  regval = rwunlockGRC();
  /* FIX -- implement SEQ locks */
  /*  regval2 = rwunlockSEQ(); */
  outbGRC((kgiu16)(page & 0xFF), GRC_PR0A);
  /*  outbSEQ(page >> 1, SR14_PR0A_B8, SEQ_FB_CONTROL); */
  outbGRC(lockreg,GRC_UPGRC_LOCK);
}


